/**
* Title: 		ServletTwo
* Description:	A simple servlet managing only POST HTTP requests
* Copyright:    Copyright (c) 2009-2011
* Company:      Morpheus
* @author 		Stefano Adriani
* @version 		1.1
*/

package adriani.webapp ;

import adriani.webapp.util.* ;

import java.io.*;
import java.util.* ;
import javax.servlet.*;
import javax.servlet.http.*;

/**
* A simple servlet mapped to the logic name "servletTwo" by the <code>web.xml</code> configuration file. 
* This class implements a simple HTML controller receiving data via the POST method. 
* Scope and mapping rules are defined in the <code>web.xml</code> configuration file. For example:<br/><br/>
* <pre><code>
*     &lt;servlet-name&gt;servletTwo&lt;/servlet-name&gt;
      &lt;url-pattern&gt;/controller&lt;/url-pattern&gt;
* </code></pre>
* Specify that this servlet will be invoked when calling the URL "/controller". In the Test web app
* enviroment this should happen when calling the URL:<br/><br/>
*
* <code>http://localhost:8080/test/controller</code>
*
*/
public class ServletTwo extends HttpServlet {

	/** CLASS ATTRIBUTES																				*/
	private HttpServletRequest request ;

	/**
	* Method automatically invoked by the Servlet Container the first time the servlet is called.
	*/
	public void init(ServletConfig config) throws ServletException {
	
		super.init(config) ;
		if (Settings.DETAILS) System.out.println("*** " +  this.getClass().getName() + " initialized...") ;
	}	
	
	/**
	* Method invoked when the servlet is activated by a normal URL request (i.e. a GET request).
	* If it's necessary to read incoming HTTP parameters or cookies use the <code>request</code> object.
	* To send data to the client browser use the <code>response</code> object.
	*
	* @param request the HTTP request (passed automatically by Tomcat).
	* @param response the HTTP response (returned automatically by Tomcat).
	*/
	public void doGet(HttpServletRequest request, HttpServletResponse response) 
		throws ServletException, IOException {
	
		PrintWriter out = response.getWriter();
		
		// Producing HTML short error message
		out.write("<html>\n") ;		
		out.write("<body>\n") ;
		out.write("<p class='Main_Font'><strong>SERVLET TWO PAGE</strong></p>\n") ;
		out.write("<p class='Main_Font'>Warning: this servlet replies only to POST request!</p>\n") ;
		out.write("<p class='Main_Font'>Please go back and try the other link (using a POST request)</p>\n") ;
		out.write("<br/>\n") ;		
		out.write("<div align='center'><a class='Small_Link' href='javascript:history.back()' title='Go back to previous page'>Back</a></div>") ;
		out.write("</body>\n") ;
		out.write("</html>\n") ;
		
		// Use following code if it's necessary to forward the request to another page
		//ServletContext context = getServletContext() ;
		//RequestDispatcher dispatcher = context.getRequestDispatcher("/home/home.jsp") ;
		//dispatcher.forward(request, response) ;
	}

	/**
	* Method invoked when the servlet is activated receiving an HTML form (i.e. a POST request).
	*
	* @param request the HTTP request (passed automatically by Tomcat).
	* @param response the HTTP response (returned automatically by Tomcat).
	*/
	public void doPost(HttpServletRequest request, HttpServletResponse response) 
		throws ServletException, IOException {
      
		this.request = request ;
		PrintWriter out = response.getWriter();
		
		// Producing XHTML header
		out.write("<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Transitional//EN' 'http://www.w3.org/TR/xhtml11-transitional.dtd'>\n") ;
		out.write("<html xmlns='http://www.w3.org/1999/xhtml' xml:lang='it' lang='it'>\n") ;
		out.write("<head>\n") ;
		out.write("<title>Web Page produced by ServletTwo</title>\n") ;
		out.write("<link href='css/style_01.css' rel='stylesheet' type='text/css'/>\n") ;
		out.write("<meta http-equiv='Content-Type' content='text/html; charset=iso-8859-1'/>\n") ;
		out.write("</head>\n") ;
		out.write("<body style='MARGIN:20px; BACKGROUND-COLOR: #DDEEFF'>\n") ;
		out.write("<p class='Main_Title' align='center'>SERVLET TWO PAGE</p>\n") ;
		out.write("<p class='Main_Font'>Hello from the <code>ServletTwo</code> class!</p>\n") ;
		
		// Producing XHTML BODY content
		int[] selection = getSelection("boxes_", "on") ;
		out.write("<p class='Main_Font'><strong>The user has selected following checkboxes:</strong></p>\n") ;
		if (selection.length > 0) for (int i = 0 ; i < selection.length; i++) {
			out.write("<p class='Main_Font'>Chekbox number " + selection[i] + "</p>\n") ;
		} else {
			out.write("<p class='Main_Font'>None</p>\n") ;
		}
		out.write("<p class='Main_Font'>For a total of:&nbsp;" + selection.length + "&nbsp;checkboxes.</p>\n") ;
		
		// Producing XHTML footer
		out.write("<br/><hr/><br/>\n") ;
		out.write("<div align='center'><a class='Small_Link' href='javascript:history.back()' title='Go back to previous page'>Back</a></div>") ;
		out.write("</body>\n") ;
		out.write("</html>\n") ;
	}
	
// ************************					UTILITIES						***********************************

    /** 
	* Retrieve an array of "boolean-like" parameters from an HTTP POST request.
    *
    * @param reference the String value common to all the income parameters (e.g. "check_").
    * @param theValue the String value bound to the "true" value (e.g "on", "yes" or something else).
    * @return the array of the boolean values.
    */
    public int[] getSelection(String reference, String theValue) {

	  // 1. Setting variables
	  int total = 0 ;
	  String key = null ;
	  String value = null ;
	  String parameter = null ;
	  int[] chosen = new int[99] ;
	  int number = reference.length() ;
	  Enumeration parameters = request.getParameterNames() ;

	  //2. Identifying the users selection
	  if (parameters != null) while (parameters.hasMoreElements()) {

	    parameter = (String) parameters.nextElement() ;
	    if (number <= parameter.length()) key = parameter.substring(0, number) ;
	    if (key != null) if (key.equals(reference)) {
		  value = request.getParameter(parameter);
		  key = parameter.substring(number, parameter.length()) ;
		  if (value.equals(theValue)) chosen[total++] = Integer.parseInt(key) ;
		  System.out.println("ServletTwo: record (" + key + "): " + value) ;				  
        }

      } // End while

	  //3. Optimizing the array size
	  // Remarl: a better code should use Vector or ArrayList (here we use very basic Java)
	  int[] result = new int[total] ;
	  for (int i = 0 ; i < total; i++) result[i] = chosen[i] ;
      return result ;
	}
  
} // END OF CLASS
