/**
* Title: Element
* Description:
* Copyright:    Copyright (c) 2009
* Company:      Morpheus
* @author Stefano Adriani
* @version 1.0
*/

package adriani.webapp.axis ;

import java.util.* ;
import javax.xml.stream.* ;
import javax.xml.namespace.* ;

import org.apache.axiom.om.* ;
import org.apache.axiom.om.impl.llom.* ;

import org.apache.axis2.databinding.* ;
import org.apache.axis2.databinding.utils.* ;
import org.apache.axis2.databinding.utils.writer.* ;
import org.apache.axis2.databinding.utils.reader.* ;

/**
* Parent class for all the classes representing a SOAP message used by the web service.
*
*/
public class Element implements ADBBean {

	/** CLASS ATTRIBUTES BY STEVE																	*/
	public String nodeValue ;
	public String elementName ;
	public static boolean VERBOSE = true ;	

	/** The leading name of each message logged on the standard output.									*/
	protected String leadingName = "Element - " ;

	/** 
	* The Qualified name for the XML element. The java classes produced by the Eclipse's WTP plugin
	* define this attribute using:<br/><br/>
	*
	* <code>public static final QName MY_QNAME = new QName("http://www.example.org/", elementName, "ns1")</code><br/><br/>
	*
	* this allows the <code>XXXMessageReceiverInOut.toOM()</code> and <code>XXXMessageReceiverInOut.toEnvelope()</code>
	* to obtain the QName reference even if no instance of the element is available. But this choice forces 
	* to use a custom ADBBean implementation for each SOAP message! To simplify this Axis2 tutorial,
	* we have modified the <code>XXXMessageReceiverInOut</code> methods in order to access only non-static
	* references of the element QName. In this way we can use this class as the parent class for all the
	* other elements classes.
	*/	
	public QName MY_QNAME ;
	
	/**
	* This constructor has been created only for training purposes, in order to ease the understanding
	* of the package structure of the classes produced by Eclipse. In the actual Axi2 paradigm THERE IS 
	* NOT such a constructor, because all the element classes are produced using the <code>Factory</code>
	* inner class (see <code>Element_1</code> and <code>Element_2</code>).
	*
	* @param name the not qualified name for this element.
	*/
	public Element(String name) {
		
		this.elementName = name ;		
		this.MY_QNAME = new QName("http://www.example.org/", elementName, "ns1") ;
		if (VERBOSE) System.out.println(leadingName + "Created " + this + " with QName: " + MY_QNAME) ;
	}

	/**
	* Method
	*
	* @param namespace the schema namespace (using the URL syntax).
	* @return the prefix associated to the namespace.
	*/
	private static String generatePrefix(String namespace) {
	
		if (namespace.equals("http://www.example.org/")) return "ns1" ;
		return BeanUtil.getUniquePrefix() ;
	}

// ************************					PUBLIC USER API 			*****************************************		
	
	/**
	* General accessor. When using the actual classes produced by Eclise, these methods are simple named
	* exactly as this class, for example <code>getElement_1()</code> or <code>getElement_2()</code>.
	*
	* @return the node content as a String object.
	*/
	public String getNodeValue()					{		return nodeValue ;								}

	/**
	* General mutator. When using the actual classes produced by Eclise, these methods are simple named
	* exactly as this class, for example <code>setElement_1()</code> or <code>setElement_2()</code>.
	*
	* @param param the text string to be used as the node content.
	*/
	public void setNodeValue(String param) 			{		this.nodeValue = param ;						}

// ************************					OBJECT SERIALIZATION		*****************************************				
	
	/**
	* isReaderMTOMAware
	* @return true if the reader supports MTOM
	*/
	public static boolean isReaderMTOMAware(XMLStreamReader reader) {
	
		boolean isReaderMTOMAware = false ;

		try{
			isReaderMTOMAware = Boolean.TRUE.equals(reader.getProperty(OMConstants.IS_DATA_HANDLERS_AWARE)) ;
		} catch(IllegalArgumentException e) {
			isReaderMTOMAware = false ;
		}
		return isReaderMTOMAware ;
	}

	/**
	* Return the <code>OMElement</code> object.<br/>
	* This method declares <em>final</em> the input parameters to ensure that these objects ARE NOT CHANGED
	* within the method's code block.
	*
	* @param parentQName
	* @param factory
	* @return OMElement
	*/
	public OMElement getOMElement(final QName parentQName, final OMFactory factory) throws ADBException {

		if (Element.VERBOSE) System.out.println(leadingName + "Creating datasource: " + MY_QNAME) ;
	
		// Defining the dataSource object, where the anonymous local class will be casted
		OMDataSource dataSource = new ADBDataSource(this, MY_QNAME) 

		/** Creating an anonymous local class on the fly, which will be cast into the dataSource object
		* (because OMDataSource it's an interface and cannot be instanciated directly).
		* Notice that the ADBDataSource it's an abstract class having only one abstract method, i.e.
		* the serialize(MTOMAwareXMLStreamWriter xmlWriter) method.
		*/		
		{
			/** The only method implemented by the anonymous local class */
			public void serialize(MTOMAwareXMLStreamWriter xmlWriter) throws XMLStreamException {

				Element.this.serialize(MY_QNAME, factory, xmlWriter) ;							
				//System.out.println("*** Inner class 1:" + this.getClass().getName() ) ;				
				//System.out.println("*** Inner class 2:" + Element.this.getClass().getName() ) ;				
			}
		} ; // End of the anonymous local class

		//System.out.println("*** Inner class 3:" + this.getClass().getName() ) ;
		//System.out.println("*** Inner class 4:" + dataSource.getClass().getName() ) ;
		//System.out.println("*** Created datasource: " + this + "-" + MY_QNAME) ;
		
		return new OMSourcedElementImpl(MY_QNAME, factory, dataSource) ;
	}

	/**
	*
	*/
	public void serialize(final QName parentQName, final OMFactory factory, MTOMAwareXMLStreamWriter xmlWriter)
		throws XMLStreamException, ADBException {
		
		serialize(parentQName, factory, xmlWriter, false) ;
	}

	/**
	*
	*/
	public void serialize(final QName parentQName, final OMFactory factory, MTOMAwareXMLStreamWriter xmlWriter,	boolean serializeType)
		throws XMLStreamException, ADBException {

		//We can safely assume an element has only one type associated with it
		String localName = elementName ;
		String namespace = "http://www.example.org/" ;
		if (Element.VERBOSE) System.out.println(leadingName + "Serializing " + this) ;		

		if (! namespace.equals("")) {
		
			String prefix = xmlWriter.getPrefix(namespace) ;
			if (prefix == null) {
				prefix = generatePrefix(namespace) ;
				xmlWriter.writeStartElement(prefix, localName, namespace) ;
				xmlWriter.writeNamespace(prefix, namespace) ;
				xmlWriter.setPrefix(prefix, namespace) ;
			} else {
				xmlWriter.writeStartElement(namespace, localName) ;
			}
		} else {
			xmlWriter.writeStartElement(localName) ;
		}

		// add the type details if this is used in a simple type
		if (serializeType) {
			String namespacePrefix = registerPrefix(xmlWriter, "http://www.example.org/") ;
			if ( (namespacePrefix != null) && (namespacePrefix.trim().length() > 0) ) {
				writeAttribute("xsi", "http://www.w3.org/2001/XMLSchema-instance", "type", namespacePrefix + ":" + elementName, xmlWriter) ;
			} else {
				writeAttribute("xsi", "http://www.w3.org/2001/XMLSchema-instance", "type", elementName, xmlWriter) ;
			}
		}

		if (nodeValue == null) throw new ADBException("Value cannot be null !!") ;
			else xmlWriter.writeCharacters(nodeValue) ;

		xmlWriter.writeEndElement() ;
	}

	/**
	* Util method to write an attribute with the ns prefix
	*/
	private void writeAttribute(String prefix, String namespace, String attName, String attValue, javax.xml.stream.XMLStreamWriter xmlWriter)
		throws XMLStreamException {

		if (xmlWriter.getPrefix(namespace) == null) {
			xmlWriter.writeNamespace(prefix, namespace) ;
			xmlWriter.setPrefix(prefix, namespace) ;
		}
		xmlWriter.writeAttribute(namespace, attName, attValue) ;
	}

	/**
	* Util method to write an attribute without the ns prefix
	*/
	private void writeAttribute(String namespace, String attName, String attValue, XMLStreamWriter xmlWriter) 
		throws XMLStreamException {
		
		if (namespace.equals("")) {
			xmlWriter.writeAttribute(attName,attValue) ;
		} else {
			registerPrefix(xmlWriter, namespace) ;
			xmlWriter.writeAttribute(namespace, attName, attValue) ;
		}
	}

	/**
	* Util method to write an attribute without the ns prefix
	*/
	private void writeQNameAttribute(String namespace, String attName, QName qname, XMLStreamWriter xmlWriter) 
		throws XMLStreamException {

		String attributeNamespace = qname.getNamespaceURI() ;
		String attributePrefix = xmlWriter.getPrefix(attributeNamespace) ;
		if (attributePrefix == null) {
			attributePrefix = registerPrefix(xmlWriter, attributeNamespace) ;
		}
		String attributeValue ;
		if (attributePrefix.trim().length() > 0) {
			attributeValue = attributePrefix + ":" + qname.getLocalPart() ;
		} else {
			attributeValue = qname.getLocalPart() ;
		}

		if (namespace.equals("")) {
			xmlWriter.writeAttribute(attName, attributeValue) ;
		} else {
			registerPrefix(xmlWriter, namespace) ;
			xmlWriter.writeAttribute(namespace, attName, attributeValue) ;
		}
	}

	/**
	*  method to handle Qnames
	*/
	private void writeQName(QName qname, XMLStreamWriter xmlWriter)	throws XMLStreamException {
	
		String namespaceURI = qname.getNamespaceURI() ;
		
		if (namespaceURI != null) {
		
			String prefix = xmlWriter.getPrefix(namespaceURI) ;
			if (prefix == null) {
				prefix = generatePrefix(namespaceURI) ;
				xmlWriter.writeNamespace(prefix, namespaceURI) ;
				xmlWriter.setPrefix(prefix,namespaceURI) ;
			}

			if (prefix.trim().length() > 0) {
				xmlWriter.writeCharacters(prefix + ":" + ConverterUtil.convertToString(qname)) ;
			} else {
				// i.e this is the default namespace (no prefix)
				xmlWriter.writeCharacters(ConverterUtil.convertToString(qname)) ;
			}
		} else {
			xmlWriter.writeCharacters(ConverterUtil.convertToString(qname)) ;
		}
	}

	private void writeQNames(QName[] qnames, XMLStreamWriter xmlWriter) throws XMLStreamException {

		if (qnames != null) {
		
			// we have to store this data until last moment since it is not possible to write any
			// namespace data after writing the character data
			String prefix = null ;		
			String namespaceURI = null ;		
			StringBuffer stringToWrite = new StringBuffer() ;

			for (int i = 0; i < qnames.length; i++) {
			
				if (i > 0) stringToWrite.append(" ") ;
				namespaceURI = qnames[i].getNamespaceURI();
				
				if (namespaceURI != null) {
					prefix = xmlWriter.getPrefix(namespaceURI) ;
					if ( (prefix == null) || (prefix.length() == 0) ) {
						prefix = generatePrefix(namespaceURI) ;
						xmlWriter.writeNamespace(prefix, namespaceURI) ;
						xmlWriter.setPrefix(prefix,namespaceURI) ;
					}
					if (prefix.trim().length() > 0) 
						stringToWrite.append(prefix).append(":").append( ConverterUtil.convertToString(qnames[i]) ) ;
					else 
						stringToWrite.append(ConverterUtil.convertToString(qnames[i]));
				} else {
					stringToWrite.append(ConverterUtil.convertToString(qnames[i]));
				}
			} // End of for cycle
			
			xmlWriter.writeCharacters(stringToWrite.toString());
		}
	}

	/**
	* Register a namespace prefix
	*/
	private String registerPrefix(XMLStreamWriter xmlWriter, String namespace) throws XMLStreamException {

		String prefix = xmlWriter.getPrefix(namespace) ;

		if (prefix == null) {
		
			prefix = generatePrefix(namespace);

			while (xmlWriter.getNamespaceContext().getNamespaceURI(prefix) != null) {
				prefix = BeanUtil.getUniquePrefix();
			}

			xmlWriter.writeNamespace(prefix, namespace);
			xmlWriter.setPrefix(prefix, namespace);
		}
		
		return prefix;
	}

	/**
	* databinding method to get an XML representation of this object
	*
	*/
	public XMLStreamReader getPullParser(QName qName) throws ADBException {

		// We can safely assume an element has only one type associated with it
		// Calling the constructor ADBXMLStreamReaderImpl(MY_QNAME, Object[], null)
		return new ADBXMLStreamReaderImpl(
			MY_QNAME,
			new java.lang.Object[] {
				ADBXMLStreamReader.ELEMENT_TEXT, ConverterUtil.convertToString(nodeValue)
			},
			null
		) ;
	}

} // END OF CLASS