/**
 * Title: MyServiceMessageReceiverInOut
 * Description:
 * Copyright:    Copyright (c) 2009
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */
 
package adriani.webapp.axis ;

import adriani.webapp.util.* ;

import java.util.* ;
import javax.xml.namespace.* ;

import org.apache.axis2.* ;
import org.apache.axis2.util.* ;
import org.apache.axis2.context.* ;
import org.apache.axis2.receivers.* ;
import org.apache.axis2.description.* ;
import org.apache.axis2.databinding.* ;

import org.apache.axiom.om.* ;
import org.apache.axiom.soap.* ;

/**
* This is the main class used by Axis to forward the web service call to the skeleton class. It retrieves
* the name of the class implementing the skleton from the service.xml file, which is retrieved by the
* <code>MessageContext</code> class in the org.apache.axis2.context package.
*
* 
*/
public class MyServiceMessageReceiverInOut extends AbstractInOutMessageReceiver {

  	/** CLASS ATTRIBUTES																				*/
	protected static boolean VERBOSE = true ;
	
	/** The leading name of each message logged on the standard output.									*/
	protected String leadingName = "MessageReceiverInOut - " ;

	/**
	*  Method
	*/
	public void invokeBusinessLogic(MessageContext msgContext, MessageContext newMsgContext) throws AxisFault {

        try {

			// Defining objects ;
			String methodName = null ;
			SOAPEnvelope envelope = null ;
			
			// Get the implementation class for the Web Service
			Object obj = getTheImplementationObject(msgContext) ;
			MyServiceSkeleton skel = (MyServiceSkeleton) obj ;
			//Find the axisOperation that has been set by the Dispatch phase.
			AxisOperation op = msgContext.getOperationContext().getAxisOperation() ;

			// If no operation is found, we just throw a new Exception
			if (op == null) {
				String msg = "Operation is not located, if this is doclit style the SOAP-ACTION should specified via the SOAP Action to use the RawXMLProvider" ;
				throw new AxisFault(msg) ;
			}

			// Targeting the invoked method
			if (op.getName() != null) methodName = JavaUtils.xmlNameToJava( op.getName().getLocalPart() ) ;
			if (methodName!= null) {

				if("Operation_01".equals(methodName)) {
					
					Element_2 element_21 = null ;
					Element_1 wrappedParam = (Element_1) fromOM(
										msgContext.getEnvelope().getBody().getFirstElement(),
										Element_1.class,
										getEnvelopeNamespaces(msgContext.getEnvelope())
										);
													
					element_21 = skel.Operation_01(wrappedParam) ;
					envelope = toEnvelope(getSOAPFactory(msgContext), element_21, false) ;
										
				} else {
				  throw new RuntimeException("method not found") ;
				}
        
				newMsgContext.setEnvelope(envelope);
			}

		// If anything goes wrong we just throw a new exception
        } catch (java.lang.Exception e) {
			throw AxisFault.makeFault(e) ;
        }
	}

// ************************			METHODS USING STATIC REFERENCE 			************************************
	
	/**
	*  Method
	*/
	private OMElement toOM(Element_1 param, boolean optimizeContent) throws AxisFault {
		
		// TUTORIAL: creating the object in order to access the not static field "qName"
		QName qName = param.MY_QNAME ;
		if (VERBOSE) System.out.println(leadingName + "Using toOM(Element_1)") ;		
		
		// Actual code block produced by Eclipse
		try{
			return param.getOMElement(qName, OMAbstractFactory.getOMFactory()) ;
		} catch(ADBException e) {
			throw AxisFault.makeFault(e) ;
		}
	}
        
	/**
	*  Method
	*/
	private OMElement toOM(Element_2 param, boolean optimizeContent) throws AxisFault {
	
		// TUTORIAL: creating the object in order to access the not static field "qName"
		QName qName = param.MY_QNAME ;
		if (VERBOSE) System.out.println(leadingName + "Using toOM(Element_2)") ;				
		
		// Actual code block produced by Eclipse
		try{
			 return param.getOMElement(qName, OMAbstractFactory.getOMFactory()) ;
		} catch(ADBException e){
			throw AxisFault.makeFault(e) ;
		}
	}
        
	/**
	*  Method
	*/
	private  SOAPEnvelope toEnvelope(SOAPFactory factory, Element_2 param, boolean optimizeContent) throws AxisFault {

		// TUTORIAL: creating the object in order to access the not static field "qName"
		QName qName = param	.MY_QNAME ;
		if (VERBOSE) System.out.println(leadingName + "Using toEnvelope()") ;				
		
		// Actual code block produced by Eclipse
		try{
			SOAPEnvelope emptyEnvelope = factory.getDefaultEnvelope() ;
			emptyEnvelope.getBody().addChild(param.getOMElement(qName, factory)) ;
			return emptyEnvelope;
		} catch(ADBException e){
			throw AxisFault.makeFault(e) ;
		}
	}
	
// **********************************			UTILITIES 			******************************************
	
	/**
	*  Method
	*/
	private Element_2 wrapOperation_01() {
	
		Element_2 wrappedElement = new Element_2() ;
		return wrappedElement ;
	}

	/**
	*  get the default envelope
	*/
	private SOAPEnvelope toEnvelope(SOAPFactory factory) {
	
		return factory.getDefaultEnvelope() ;
	}

	/**
	*  Method
	*/
	private Object fromOM(OMElement param, Class type, Map extraNamespaces) throws AxisFault {

        try {
			if (Element_1.class.equals(type)) {
				if (Element.VERBOSE) System.out.println(leadingName + "Invoking Element_1.Factory") ;
				return Element_1.Factory.parse(param.getXMLStreamReaderWithoutCaching()) ;
			}
	   
			if (Element_2.class.equals(type)) {
				if (Element.VERBOSE) System.out.println(leadingName + "Invoking Element_2.Factory") ;			
				return Element_2.Factory.parse(param.getXMLStreamReaderWithoutCaching()) ;
			}
        } catch (java.lang.Exception e) {
			throw AxisFault.makeFault(e) ;
        }
		return null ;
	}

	/**
	*  A utility method that copies the namepaces from the SOAPEnvelope
	*/
	private Map getEnvelopeNamespaces(SOAPEnvelope env) {
	
        Map returnMap = new java.util.HashMap() ;
        Iterator namespaceIterator = env.getAllDeclaredNamespaces() ;
        while (namespaceIterator.hasNext()) {
			OMNamespace ns = (OMNamespace) namespaceIterator.next() ;
			returnMap.put(ns.getPrefix(), ns.getNamespaceURI()) ;
        }
        return returnMap ;
	}

	/**
	*  Method
	*/
	private AxisFault createAxisFault(java.lang.Exception e) {
	
		AxisFault f ;
		Throwable cause = e.getCause() ;
		if (cause != null) f = new AxisFault(e.getMessage(), cause) ;
		else f = new AxisFault( e.getMessage() ) ;
		if (VERBOSE) System.out.println(leadingName + "Created new axis fault") ;
		return f ;
	}

} // End of class
    