/**
* Title: 		SimpleJAXWS
* Description:	A simple web service created using the JAX-WS annotation approach, based on the Java5 features
* Copyright:    Copyright (c) 2011-2012
* Company:      Morpheus
* @author 		Stefano Adriani
* @version 		1.2
*/

package adriani.webapp.jaxws ;

import java.util.Date ;

import javax.jws.WebMethod ;
import javax.jws.WebService ;
import javax.annotation.Resource;
import javax.xml.ws.WebServiceContext ;

/**
* A simple web service created using the JAX-WS annotations approach, based on the Java5 features.<br/>
* This class is defined using the <code>@WebService</code> annotation before the class definition, and 
* <code>@WebMethod</code> annotation before the each method to be offered by the web service.
* <br/><br/>
* Notice that it's necessary to import the packages of the annotations, since the annotations used here
* are not default Java annotations, i.e. they are not distributed with the default JDK enviroment. 
* For the same reason, it could be necessary to include the <code>axis2-jaxws</code> library 
* (for example <code>axis2-jaxws-1.5.4.jar</code>) in the compile classpath to succesfully compile this class.
*/
@WebService
public class SimpleJAXWS {

	/** If <code>true</code> very detailed messages are printed on the STDOUT							*/
	private 			boolean 			verbose 		= 		true ;

	/** The leading string printed before any message logged by this class	 							*/
	private static		String	 			leadingName		= 		"SimpleJAX-WS - " ;
    
	/** The @Resource annotation, used for resource injection, is defined by the JSR-250 specification	*/
    @Resource
    private WebServiceContext ctx;

	/**
	* FIRST METHOD: a simple "echo" method
	*
	* @param input a string
	* @return the echoed input string, for example "SimpleJAX-WS input"
	*/
	@WebMethod 
    public String echo(String input) {
	
        if (verbose) System.out.println(leadingName + "Invoked on " + new Date()) ;
		return leadingName + input ;
    }
	
	/**
	* SECOND METHOD: a simple double calculation method
	*
	* @param x a double value
	* @param y a double value
	* @return the sum of the two input double values
	*/
	@WebMethod 
	public double add(double x, double y) {
	
		return x + y ;
	}	
     
} // END OF CLASS