 /**
 * Title: ClientThread
 * Description:
 * Copyright:    Copyright (c) 2009
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.webapp.util ;

/**
* Class implementing the <code>Runnable</code> interface and used by the <code>test_concurrent.jsp</code> page
* to verify the behaviour or web services under concurrent invocations.
*/
public class ClientThread implements Runnable {

	/** Thread of this runnable object																		*/
	public Thread theThread ;

	/** The SOAP client calling the service	using HTTP transport											*/
	public SimpleHTTPClient client ;

	/** The URL of the target server hosting the web service (EPR).											*/
	public String server ;

	/** The string value to be used for the soap input request.												*/
	public String param ;
	
	/** The string value to be used for the soap output response.											*/
	public StringBuffer response ;
	
	/** The integer index of this Thread, as created by the JSP page.										*/
	public int index ;

	/** The leading name of each message logged on the STDOUT.												*/
	protected String leadingName = "ClientThread - " ;

	/**
	* Standart constructor for this class.
	*
	* @param index the int code used to identy this thread.
	*/
    public ClientThread(int index) {

		this.index = index ;
		//System.out.println(leadingName + "Created:" + index) ;
    }

	/**
	* Initialize this thread and set parameters in the SOAP client.
	*
	* @param server the URL of the target server hosting the web service (EPR).
	* @param ns the namespace used to define the XML elements in the service's WSDL.
	* @param tag_1 the opening tag of the input element.
	* @param tag_2 the closing tag of the output element.
	* @param param the string value to be used for the main request element.
	*/
    public void init(String server, String ns, String tag_1, String tag_2, String param) {

		this.param = param ;
		this.server = server ;
		this.client = new SimpleHTTPClient() ;
		this.client.init(ns, tag_1, tag_2) ;
		//System.out.println(leadingName + "Initialized:" + index) ;
	}

	/**
	* This method is invoked by the JSP page by calling <code>client[i].theThread.start()</code>: it invokes
	* the web service using the <code>SimpleHTTPClient</code> class and then exit.
	*/
    public void run() {

		//this.client.printRequest(param) ;
		response = client.call(server, param) ;
		if (response != null) System.out.println(leadingName + "Thread " + index + " received " + response.length() + " characters") ;
			else System.out.println(leadingName + "Thread " + index + " received a null response") ;
    }
	
// **********************************			ACCESSORS		*************************************************	

	/**
	* Return the response obtained by this thread.
	*/
    public StringBuffer getResponse() 		{		return this.response ;										}

}	// End of class
